@@ -8,8 +8,12 @@ pub mod client;
 /// Handlers for all incoming data
 pub mod worker;
 
+/// Authorization state handlers.
+pub mod auth_handler;
+
 #[doc(hidden)]
 pub mod tdlib_client;
 
-pub use client::Client;
-pub use worker::{AuthStateHandler, ConsoleAuthStateHandler, Worker, WorkerBuilder};
+pub use auth_handler::{AuthStateHandler, ConsoleAuthStateHandler, SignalAuthStateHandler};
+pub use client::{Client, ClientState};
+pub use worker::{Worker, WorkerBuilder};
